<?php
/**
 * Plugin Name: Asset availabilty customization
 * Plugin URI: https://pluginhive.com/
 * Description: Block asset availability if the asset is already booked in product X, but still the product X will be able to book.
 * Version: 1.0.6
 * Author: Varun, PluginHive
 * Author URI: https://pluginhive.com
 */

function ph_bookings_activation_check_for_asset_availability_customization_addon(){

    if ( !is_plugin_active('ph-bookings-appointments-woocommerce-premium/ph-bookings-appointments-woocommerce-premium.php') ){
        deactivate_plugins( basename( __FILE__ ) );
        wp_die( __("Oops! You tried installing the Asset availabilty customization Plugin without activating Bookings and Appointments For WooCommerce Premium Plugin. Kindly activate <b>Bookings and Appointments For WooCommerce Premium Plugin</b> and then try again", "bookings-and-appointments-for-woocommerce" ), "", array('back_link' => 1 ));
    }
}

register_activation_hook( __FILE__, 'ph_bookings_activation_check_for_asset_availability_customization_addon' );

//check if woocommerce exists
if ( !class_exists( 'woocommerce' ) || !class_exists('phive_booking_initialze_premium') ) {
    add_action( 'admin_init', 'Ph_Bookings_Asset_Availability_Customization' );
    if ( ! function_exists( 'Ph_Bookings_Asset_Availability_Customization' ) ) {
        function Ph_Bookings_Asset_Availability_Customization() {
            if ( !class_exists( 'woocommerce' ) || !class_exists('phive_booking_initialze_premium') ){
                deactivate_plugins( plugin_basename( __FILE__ ) );
                wp_safe_redirect( admin_url('plugins.php') );

            }
        }
    }
}
if( ! class_exists('Ph_Bookings_Assets_Availability_Customization') ) {
	class Ph_Bookings_Assets_Availability_Customization {
		public function __construct() {
			add_filter( 'ph_bookings_get_assets_availabibility', array( $this, 'modify_asset_availability'), 10, 6 );
			
			// 96421
			add_filter('ph_modify_asset_availability_when_using_availability_table', array($this, 'modify_asset_availability_when_using_availability_table'), 10, 7);
		}

		public function modify_asset_availability( $availability, $asset_id, $booking_count, $asset_max_quantity, $from, $to ) {
			$product_id_arr = array();		// Product Ids Leave Empty to apply to all
			global $product;

			if( ! empty($asset_id) ) {
				if( is_ajax() && ! empty($_POST['product_id']) ) {
					$product = wc_get_product($_POST['product_id']);
				}

				if( is_a( $product, 'WC_Product') ) {
					$product_id = $product->get_id();
					if( in_array($product_id, $product_id_arr) || empty($product_id_arr) ) {
						if( empty($this->id) || $this->id != $asset_id ) {
							$this->id 					= $asset_id;
							$this->serialized_asset_id	= serialize( array($this->id) );
							$this->booked_dates = $this->get_all_bookings();
						}
						$asset_exist_in_other_product = $this->check_asset_existance_in_other_product( $from, $to, $product_id);

						// If Asset exist in any other product then return availability of asset as zero
						if( $asset_exist_in_other_product ) {
							$availability = 0;
						}
					}
				}
			}
			return $availability;
		}

		private function check_asset_existance_in_other_product( $date, $date_to, $product_id )
		{
			$interval_period	= get_post_meta( $product_id, "_phive_book_interval_period", 1 );
			$found = 0;
			$asset_exist_in_other_product = false;
			foreach ( $this->booked_dates as $order_item_id => $booked_detail ) {
				
				switch ( strlen($date) ) {
					case 7:
						$from 	= substr($booked_detail['from'], 0, 7);
						$to 	= isset($booked_detail['to']) ? substr($booked_detail['to'], 0, 7) : substr( $this->generate_order_item_booked_to($order_item_id, $from, $booked_detail['meta_type']), 0, 7);
						break;
					
					case 10:
						$from 	= isset($booked_detail['from']) ? substr($booked_detail['from'], 0, 10) : '';
						$to 	= isset($booked_detail['to']) ? substr($booked_detail['to'], 0, 10) : substr( $this->generate_order_item_booked_to($order_item_id, $from, $booked_detail['meta_type']), 0, 10 );
						break;
					
					default:
						$from 	= isset($booked_detail['from']) ? $booked_detail['from'] : '';
						//ticket#105393
						$to 	= (isset($booked_detail['to']) && $booked_detail['to']!=$booked_detail['from']) ? $booked_detail['to'] : $this->generate_order_item_booked_to($order_item_id, $from, $booked_detail['meta_type']);
						break;
				}

				if (($interval_period == 'hour' || $interval_period == 'minute' )) 
				{
					$from = (!empty($from) && strlen($from) == 10) ? $from." 00:00" : $from;
					$to = (!empty($to) && strlen($to) == 10) ? $to." 23:59" : $to;
				}
						
				if( (!empty($from) && !empty($to) && strtotime($date) >= strtotime($from) && strtotime($date) <= strtotime($to) ) || (strtotime($date) <= strtotime($from) && strtotime($from) < strtotime($date_to) ) || (strtotime($date) <= strtotime($to) && strtotime($to) < strtotime($date_to) ) )
				{
					if( $booked_detail['product_id'] != $product_id ) {
						$asset_exist_in_other_product = true;
					}
				}
			}
			return $asset_exist_in_other_product;
			
		}

		/**
		 * Generate Booked To Date.
		 * @param string Order Item Id.
		 * @param string $from Book From Time.
		 * @return string
		 */
		private function generate_order_item_booked_to($order_item_id, $from, $meta_type){
			$product_id = '';
			if ($meta_type == 'from_post_meta') 
			{
				$product_id 		= get_post_meta($order_item_id,'_product_id',1);
			}
			elseif ($meta_type == 'from_order_item_meta') 
			{
				$product_id 		= wc_get_order_item_meta($order_item_id,'_product_id',1);
			}
			
			$interval_period	= get_post_meta( $product_id, "_phive_book_interval_period", 1 );
			$interval			= get_post_meta( $product_id, "_phive_book_interval", 1 );
			
			return date( 'Y-m-d H:i', strtotime( "+$interval $interval_period", strtotime($from) ) -1 ); //Substracting 1 second for not matching with starting time of next slot
		}

		/**
		 * Get Array of Bookings for the given assets.
		 * @return array
		 */
		private function get_all_bookings(){
			global $wpdb;
			//Query for getting booked dates
			$query = "SELECT meta_key , meta_value , order_item_id, 'from_order_item_meta' as meta_type
				FROM {$wpdb->prefix}woocommerce_order_itemmeta AS t1
				WHERE t1.order_item_id
				IN (
					SELECT order_item_id
					FROM {$wpdb->prefix}woocommerce_order_itemmeta
					WHERE	meta_key = 'Assets'
					AND	(meta_value = '$this->serialized_asset_id' OR meta_value = '$this->id')
				)AND ( 
					meta_key = 'From'
					OR meta_key = 'To'
					OR meta_key = 'canceled'
					OR meta_key = 'person_as_booking'
					OR meta_key = '_product_id'
					OR meta_key = 'Number of persons'
				)
				ORDER BY t1.order_item_id DESC LIMIT 0,1000";
			$booked = $wpdb->get_results( $query, OBJECT );
		
			//Query for getting freezed dates
			$query_post = "SELECT meta_key, meta_value, post_id as order_item_id, 'from_post_meta' as meta_type
				FROM {$wpdb->prefix}postmeta AS t1
				WHERE t1.post_id
				IN (
					SELECT post_id
					FROM  {$wpdb->prefix}postmeta
					WHERE meta_key = 'asset_id'
					AND meta_value = '$this->id'
				)AND ( 
					meta_key = 'From'
					OR meta_key = 'To'
					OR meta_key = 'person_as_booking'
					OR meta_key = '_product_id'
					OR meta_key = 'Number of persons'
				)
				AND t1.post_id 
				
				ORDER BY  t1.post_id DESC LIMIT 0,1000";
			$freezed = $wpdb->get_results( $query_post, OBJECT );
			$booked_array = array_merge( $booked, $freezed );
			$processed = array();
			$canceled = array();
			foreach ($booked_array as $key => $value) {
				
				if( $value->meta_key == 'From' ){
					$from_date = substr($value->meta_value, 0, 10);
					$processed[ $value->order_item_id ]['from'] = ph_maybe_unserialize($value->meta_value);
					$processed[ $value->order_item_id ]['meta_type'] = $value->meta_type;
				}
				if( $value->meta_key=='To' ){
					$processed[ $value->order_item_id ]['to'] = ph_maybe_unserialize($value->meta_value);
				}
				if( $value->meta_key == 'Number of persons' ){
					$number_of_person = substr($value->meta_value, 0, 10);
					$processed[ $value->order_item_id ]['Number of persons'] = $value->meta_value;
					$processed[ $value->order_item_id ]['meta_type'] = $value->meta_type;
				}
				if( $value->meta_key == 'person_as_booking' ){
					$person_as_booking = substr($value->meta_value, 0, 10);
					$processed[ $value->order_item_id ]['person_as_booking'] = $value->meta_value;
					$processed[ $value->order_item_id ]['meta_type'] = $value->meta_type;
				}
				if( $value->meta_key == '_product_id' ){
					$processed[ $value->order_item_id ]['product_id'] = $value->meta_value;
				}
				if( $value->meta_key == 'canceled' && $value->meta_value == 'yes' ){
						$canceled[$value->order_item_id] = '';
				}
			}
			
			$eliminated_cancelled =  array_diff_key($processed, $canceled);
		
			//if TO is missing, concider FROM as TO
			foreach ($eliminated_cancelled as $key => &$value) {
				if( empty($value['to']) && !empty($value['from'])){ // in the case of buffer, index 'from' wil be empty
					$value['to'] = $value['from'];
				}
			}
			
			return $eliminated_cancelled;
		}

		// 96421
		public function modify_asset_availability_when_using_availability_table($asset_left_count, $asset_id, $booking_count, $max_quantity, $current_time, $product_id, $ignore_freezed)
		{
			$product_id_arr = array();		// Product Ids Leave Empty to apply to all

			if( !empty($asset_id) )
			{
				$product = wc_get_product($product_id);

				if( is_a( $product, 'WC_Product') )
				{
					if( in_array($product_id, $product_id_arr) || empty($product_id_arr) ) 
					{
						$asset_exist_in_other_product = $this->ph_check_asset_existence_in_other_products($asset_left_count, $asset_id, $booking_count, $max_quantity, $current_time, $product_id, $ignore_freezed);

						// If Asset exist in any other product then return availability of asset as zero
						if( $asset_exist_in_other_product ) {
							$asset_left_count = 0;
						}
					}
				}
			}

			return $asset_left_count;
		}

		// 96421
		public function ph_check_asset_existence_in_other_products($asset_left_count, $asset_id, $booking_count, $max_quantity, $current_time, $product_id, $ignore_freezed)
		{
			global $wpdb;
			$tablename 			= $wpdb->prefix.'ph_bookings_availability_calculation_data';

			// Settings
			$settings 			= Ph_Booking_Manage_Availability_Data::ph_get_product_settings($product_id);

			$asset_settings 	= get_option( 'ph_booking_settings_assets', 1 );
			$rules 				= ((!empty($asset_settings)) && (isset($asset_settings['_phive_booking_assets']))) ? $asset_settings['_phive_booking_assets'] : array();
			$max_quantity 		= isset($rules[$asset_id]['ph_booking_asset_quantity']) ? (float) $rules[$asset_id]['ph_booking_asset_quantity'] : null;
			$charge_per_night	= $settings['charge_per_night'];

			$compare_date 		= date('Y-m-d H:i:s', $current_time);
			$compare_date_to	= $compare_date;
			if(($settings['interval_period'] == 'hour' || $settings['interval_period'] == 'minute'))
			{
				$interval_string 	= "+".$settings['interval']." ".$settings['interval_period'];
				$compare_date_to 	= date('Y-m-d H:i:s', strtotime($interval_string, $current_time));
			}

			// 168981 - WPML Compatibility
			$product_ids  		= array($product_id);
			$trid 				= apply_filters( 'wpml_element_trid', '', $product_id, 'post_product');
			$translations 		= apply_filters( 'wpml_get_element_translations', '', $trid, 'post_product');
			if(!empty($translations) && is_array($translations)){
				$product_ids 	= wp_list_pluck($translations, 'element_id');
			}
			$product_ids 		= array_filter($product_ids);		// in case of incomplete translations, wpml returns empty ids which causes fatal error in below query.
			$product_ids 		= implode(",",$product_ids);
			// 168981 End

			if($ignore_freezed)
			{
				$query 		= "SELECT sno, asset_id, booked_date, booked_date_end, COUNT(*) as booking_count, SUM(participant_count) as participant_count, participant_as_booking FROM $tablename where 
							booking_status    != 'canceled' AND
							booking_status    != 'refunded' AND
							booking_type	  != 'cart'		AND
							asset_id 		  = '$asset_id' AND
							DATE(booked_date) = '$compare_date' AND
							( 
								charge_per_night != 'yes'
								OR 
								booked_date_type != 'to'
							)";
				
				
				if(($settings['interval_period'] == 'hour' || $settings['interval_period'] == 'minute'))
				{
					$query 	= "SELECT sno, booked_date, COUNT(*) as booking_count, SUM(participant_count) as participant_count, participant_as_booking FROM $tablename where 
							booking_status   != 'canceled' AND
							booking_status   != 'refunded' AND
							booking_type	  != 'cart'		AND
							asset_id 		 = '$asset_id' AND
							(
								(
									booked_date 	<= '$compare_date' AND
									booked_date_end > '$compare_date'
								) OR
								(
									booked_date 	>= '$compare_date' AND
									booked_date 	<  '$compare_date_to'
								) OR
								(
									booked_date_end > '$compare_date' AND
									booked_date_end <  '$compare_date_to'
								)
							) AND
							( 
								charge_per_night != 'yes'
								OR 
								booked_date_type != 'to'
							)";
				}
			}
			else
			{
				$query 		= "SELECT sno, asset_id, booked_date, booked_date_end, COUNT(*) as booking_count, SUM(participant_count) as participant_count, participant_as_booking FROM $tablename where 
							booking_status    != 'canceled' AND
							booking_status    != 'refunded' AND
							asset_id 		  = '$asset_id' AND
							DATE(booked_date) = '$compare_date' AND
							( 
								charge_per_night != 'yes'
								OR 
								booked_date_type != 'to'
							)";

				if($settings['interval_period'] == 'hour' || $settings['interval_period'] == 'minute')
				{
					$query 	= "SELECT sno, booked_date, COUNT(*) as booking_count, SUM(participant_count) as participant_count, participant_as_booking FROM $tablename where 
							booking_status   != 'canceled' AND
							booking_status   != 'refunded' AND
							asset_id 		 = '$asset_id' AND
							(
								(
									booked_date 	<= '$compare_date' AND
									booked_date_end > '$compare_date'
								) OR
								(
									booked_date 	>= '$compare_date' AND
									booked_date 	<  '$compare_date_to'
								) OR
								(
									booked_date_end > '$compare_date' AND
									booked_date_end <  '$compare_date_to'
								)
							)AND
							(
								charge_per_night != 'yes'
								OR
								booked_date_type != 'to'
							)";
				}
			}

			// do not check for current product
			$query 	.= "AND product_id Not IN($product_ids) ";
			$query .= "GROUP BY participant_as_booking";

			$results 		= $wpdb->get_results($query);
			$booking_count 	= 0;
			foreach ($results as $key => $value) 
			{
				if($value->participant_as_booking == 'yes')
				{
					$booking_count += is_numeric($value->participant_count) ? $value->participant_count : $value->booking_count;
				}
				else
				{
					$booking_count += $value->booking_count;
				}
			}

			if($booking_count > 0)
			{
				return true;
			}
			return false;
		}

	}
}
new Ph_Bookings_Assets_Availability_Customization;